use std::env;
use std::fs::File;
use std::path::Path;
use nbt::{Blob, Value::{self, Compound}, Map};

fn read_nbt(path_str: &String) -> Option<Blob> {
    let path_to_dat = Path::new(&path_str);

    if let Ok(mut file) = File::open(path_to_dat) {
        Some(Blob::from_gzip_reader(&mut file).expect("invalid nbt format"))
    } else {
        None
    }
}

fn read_key(map: &Map<String,Value>, key: &str) -> Option<Value> {
    let value = key
        .split('.')
        .enumerate()
        .fold(
            None, // Option<&V>
            |last_value, (index, part)| {
                if index == 0 {
                    return map.get(part);
                }
                return match last_value {
                    None => None,
                    Some(Compound(inner)) => inner.get(part),
                    _ => None
                };
            }
        );
    value.cloned()
}

fn print_value_map(map: &Map<String, Value>, level: &usize) {
    for (key, value) in map.iter() {
        match value {
            Value::Compound(inner_map) => {
                print_formatted_key(key, level);
                print_value_map(inner_map, &(*level + 1));
            },
            _ => print_formatted_key(key, level)
        }
    }
}

fn print_formatted_key(key: &str, level: &usize) {
    println!("{space:>count$}{name}", space="", count=*level*4, name=key)
}

fn main() {
    let cmd_name = env::args().take(1).next().unwrap_or("nbt_reader".to_string());
    let usage = format!("Usage: {name} with .dat", name=cmd_name);
    let file_name = env::args().take(2).nth(1).expect(&usage);

    let nbt_blob = read_nbt(&file_name).unwrap_or_else(|| panic!("no valid nbt found at {path}", path=file_name));

    let key_name = env::args().take(3).nth(2);

    if let Some(nbt_key) = key_name {
        let value = read_key(&nbt_blob.content, nbt_key.as_str());
        println!("{}", value.expect("key does not exist"));
    } else {
        print_value_map(&nbt_blob.content, &0);
    }

}

